rails

package module
v0.3.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 11, 2026 License: Apache-2.0 Imports: 18 Imported by: 0

README

Rails Go API Library

Go Reference

The Rails Go library provides convenient access to the Rails REST API from applications written in Go.

It is generated with Stainless.

Installation

import (
	"github.com/railsinfra/rails-go" // imported as rails
)

Or to pin the version:

go get -u 'github.com/railsinfra/rails-go@v0.3.0'

Requirements

This library requires Go 1.22+.

Usage

The full API of this library can be found in api.md.

package main

import (
	"context"
	"fmt"

	"github.com/railsinfra/rails-go"
	"github.com/railsinfra/rails-go/option"
)

func main() {
	client := rails.NewClient(
		option.WithAPIKey("My API Key"),    // defaults to os.LookupEnv("RAILS_API_KEY")
		option.WithEnvironmentProduction(), // defaults to option.WithEnvironmentStaging()
	)
	user, err := client.Users.New(context.TODO(), rails.UserNewParams{
		Email:        "jane@example.com",
		FirstName:    "Jane",
		LastName:     "Doe",
		Password:     "your-secure-password",
		XEnvironment: rails.UserNewParamsXEnvironmentSandbox,
	})
	if err != nil {
		panic(err.Error())
	}
	fmt.Printf("%+v\n", user.UserID)
}

Request fields

The rails library uses the omitzero semantics from the Go 1.24+ encoding/json release for request fields.

Required primitive fields (int64, string, etc.) feature the tag `api:"required"`. These fields are always serialized, even their zero values.

Optional primitive types are wrapped in a param.Opt[T]. These fields can be set with the provided constructors, rails.String(string), rails.Int(int64), etc.

Any param.Opt[T], map, slice, struct or string enum uses the tag `json:"...,omitzero"`. Its zero value is considered omitted.

The param.IsOmitted(any) function can confirm the presence of any omitzero field.

p := rails.ExampleParams{
	ID:   "id_xxx",            // required property
	Name: rails.String("..."), // optional property

	Point: rails.Point{
		X: 0,            // required field will serialize as 0
		Y: rails.Int(1), // optional field will serialize as 1
		// ... omitted non-required fields will not be serialized
	},

	Origin: rails.Origin{}, // the zero value of [Origin] is considered omitted
}

To send null instead of a param.Opt[T], use param.Null[T](). To send null instead of a struct T, use param.NullStruct[T]().

p.Name = param.Null[string]()       // 'null' instead of string
p.Point = param.NullStruct[Point]() // 'null' instead of struct

param.IsNull(p.Name)  // true
param.IsNull(p.Point) // true

Request structs contain a .SetExtraFields(map[string]any) method which can send non-conforming fields in the request body. Extra fields overwrite any struct fields with a matching key. For security reasons, only use SetExtraFields with trusted data.

To send a custom value instead of a struct, use param.Override[T](value).

// In cases where the API specifies a given type,
// but you want to send something else, use [SetExtraFields]:
p.SetExtraFields(map[string]any{
	"x": 0.01, // send "x" as a float instead of int
})

// Send a number instead of an object
custom := param.Override[rails.FooParams](12)
Request unions

Unions are represented as a struct with fields prefixed by "Of" for each of its variants, only one field can be non-zero. The non-zero field will be serialized.

Sub-properties of the union can be accessed via methods on the union struct. These methods return a mutable pointer to the underlying data, if present.

// Only one field can be non-zero, use param.IsOmitted() to check if a field is set
type AnimalUnionParam struct {
	OfCat *Cat `json:",omitzero,inline`
	OfDog *Dog `json:",omitzero,inline`
}

animal := AnimalUnionParam{
	OfCat: &Cat{
		Name: "Whiskers",
		Owner: PersonParam{
			Address: AddressParam{Street: "3333 Coyote Hill Rd", Zip: 0},
		},
	},
}

// Mutating a field
if address := animal.GetOwner().GetAddress(); address != nil {
	address.ZipCode = 94304
}
Response objects

All fields in response structs are ordinary value types (not pointers or wrappers). Response structs also include a special JSON field containing metadata about each property.

type Animal struct {
	Name   string `json:"name,nullable"`
	Owners int    `json:"owners"`
	Age    int    `json:"age"`
	JSON   struct {
		Name        respjson.Field
		Owner       respjson.Field
		Age         respjson.Field
		ExtraFields map[string]respjson.Field
	} `json:"-"`
}

To handle optional data, use the .Valid() method on the JSON field. .Valid() returns true if a field is not null, not present, or couldn't be marshaled.

If .Valid() is false, the corresponding field will simply be its zero value.

raw := `{"owners": 1, "name": null}`

var res Animal
json.Unmarshal([]byte(raw), &res)

// Accessing regular fields

res.Owners // 1
res.Name   // ""
res.Age    // 0

// Optional field checks

res.JSON.Owners.Valid() // true
res.JSON.Name.Valid()   // false
res.JSON.Age.Valid()    // false

// Raw JSON values

res.JSON.Owners.Raw()                  // "1"
res.JSON.Name.Raw() == "null"          // true
res.JSON.Name.Raw() == respjson.Null   // true
res.JSON.Age.Raw() == ""               // true
res.JSON.Age.Raw() == respjson.Omitted // true

These .JSON structs also include an ExtraFields map containing any properties in the json response that were not specified in the struct. This can be useful for API features not yet present in the SDK.

body := res.JSON.ExtraFields["my_unexpected_field"].Raw()
Response Unions

In responses, unions are represented by a flattened struct containing all possible fields from each of the object variants. To convert it to a variant use the .AsFooVariant() method or the .AsAny() method if present.

If a response value union contains primitive values, primitive fields will be alongside the properties but prefixed with Of and feature the tag json:"...,inline".

type AnimalUnion struct {
	// From variants [Dog], [Cat]
	Owner Person `json:"owner"`
	// From variant [Dog]
	DogBreed string `json:"dog_breed"`
	// From variant [Cat]
	CatBreed string `json:"cat_breed"`
	// ...

	JSON struct {
		Owner respjson.Field
		// ...
	} `json:"-"`
}

// If animal variant
if animal.Owner.Address.ZipCode == "" {
	panic("missing zip code")
}

// Switch on the variant
switch variant := animal.AsAny().(type) {
case Dog:
case Cat:
default:
	panic("unexpected type")
}
RequestOptions

This library uses the functional options pattern. Functions defined in the option package return a RequestOption, which is a closure that mutates a RequestConfig. These options can be supplied to the client or at individual requests. For example:

client := rails.NewClient(
	// Adds a header to every request made by the client
	option.WithHeader("X-Some-Header", "custom_header_info"),
)

client.Users.New(context.TODO(), ...,
	// Override the header
	option.WithHeader("X-Some-Header", "some_other_custom_header_info"),
	// Add an undocumented field to the request body, using sjson syntax
	option.WithJSONSet("some.json.path", map[string]string{"my": "object"}),
)

The request option option.WithDebugLog(nil) may be helpful while debugging.

See the full list of request options.

Pagination

This library provides some conveniences for working with paginated list endpoints.

You can use .ListAutoPaging() methods to iterate through items across all pages:

Or you can use simple .List() methods to fetch a single page and receive a standard response object with additional helper methods like .GetNextPage(), e.g.:

Errors

When the API returns a non-success status code, we return an error with type *rails.Error. This contains the StatusCode, *http.Request, and *http.Response values of the request, as well as the JSON of the error body (much like other response objects in the SDK).

To handle errors, we recommend that you use the errors.As pattern:

_, err := client.Users.New(context.TODO(), rails.UserNewParams{
	Email:        "jane@example.com",
	FirstName:    "Jane",
	LastName:     "Doe",
	Password:     "your-secure-password",
	XEnvironment: rails.UserNewParamsXEnvironmentSandbox,
})
if err != nil {
	var apierr *rails.Error
	if errors.As(err, &apierr) {
		println(string(apierr.DumpRequest(true)))  // Prints the serialized HTTP request
		println(string(apierr.DumpResponse(true))) // Prints the serialized HTTP response
	}
	panic(err.Error()) // GET "/api/v1/users": 400 Bad Request { ... }
}

When other errors occur, they are returned unwrapped; for example, if HTTP transport fails, you might receive *url.Error wrapping *net.OpError.

Timeouts

Requests do not time out by default; use context to configure a timeout for a request lifecycle.

Note that if a request is retried, the context timeout does not start over. To set a per-retry timeout, use option.WithRequestTimeout().

// This sets the timeout for the request, including all the retries.
ctx, cancel := context.WithTimeout(context.Background(), 5*time.Minute)
defer cancel()
client.Users.New(
	ctx,
	rails.UserNewParams{
		Email:        "jane@example.com",
		FirstName:    "Jane",
		LastName:     "Doe",
		Password:     "your-secure-password",
		XEnvironment: rails.UserNewParamsXEnvironmentSandbox,
	},
	// This sets the per-retry timeout
	option.WithRequestTimeout(20*time.Second),
)
File uploads

Request parameters that correspond to file uploads in multipart requests are typed as io.Reader. The contents of the io.Reader will by default be sent as a multipart form part with the file name of "anonymous_file" and content-type of "application/octet-stream".

The file name and content-type can be customized by implementing Name() string or ContentType() string on the run-time type of io.Reader. Note that os.File implements Name() string, so a file returned by os.Open will be sent with the file name on disk.

We also provide a helper rails.File(reader io.Reader, filename string, contentType string) which can be used to wrap any io.Reader with the appropriate file name and content type.

Retries

Certain errors will be automatically retried 2 times by default, with a short exponential backoff. We retry by default all connection errors, 408 Request Timeout, 409 Conflict, 429 Rate Limit, and >=500 Internal errors.

You can use the WithMaxRetries option to configure or disable this:

// Configure the default for all requests:
client := rails.NewClient(
	option.WithMaxRetries(0), // default is 2
)

// Override per-request:
client.Users.New(
	context.TODO(),
	rails.UserNewParams{
		Email:        "jane@example.com",
		FirstName:    "Jane",
		LastName:     "Doe",
		Password:     "your-secure-password",
		XEnvironment: rails.UserNewParamsXEnvironmentSandbox,
	},
	option.WithMaxRetries(5),
)
Accessing raw response data (e.g. response headers)

You can access the raw HTTP response data by using the option.WithResponseInto() request option. This is useful when you need to examine response headers, status codes, or other details.

// Create a variable to store the HTTP response
var response *http.Response
user, err := client.Users.New(
	context.TODO(),
	rails.UserNewParams{
		Email:        "jane@example.com",
		FirstName:    "Jane",
		LastName:     "Doe",
		Password:     "your-secure-password",
		XEnvironment: rails.UserNewParamsXEnvironmentSandbox,
	},
	option.WithResponseInto(&response),
)
if err != nil {
	// handle error
}
fmt.Printf("%+v\n", user)

fmt.Printf("Status Code: %d\n", response.StatusCode)
fmt.Printf("Headers: %+#v\n", response.Header)
Making custom/undocumented requests

This library is typed for convenient access to the documented API. If you need to access undocumented endpoints, params, or response properties, the library can still be used.

Undocumented endpoints

To make requests to undocumented endpoints, you can use client.Get, client.Post, and other HTTP verbs. RequestOptions on the client, such as retries, will be respected when making these requests.

var (
    // params can be an io.Reader, a []byte, an encoding/json serializable object,
    // or a "…Params" struct defined in this library.
    params map[string]any

    // result can be an []byte, *http.Response, a encoding/json deserializable object,
    // or a model defined in this library.
    result *http.Response
)
err := client.Post(context.Background(), "/unspecified", params, &result)
if err != nil {
    …
}
Undocumented request params

To make requests using undocumented parameters, you may use either the option.WithQuerySet() or the option.WithJSONSet() methods.

params := FooNewParams{
    ID:   "id_xxxx",
    Data: FooNewParamsData{
        FirstName: rails.String("John"),
    },
}
client.Foo.New(context.Background(), params, option.WithJSONSet("data.last_name", "Doe"))
Undocumented response properties

To access undocumented response properties, you may either access the raw JSON of the response as a string with result.JSON.RawJSON(), or get the raw JSON of a particular field on the result with result.JSON.Foo.Raw().

Any fields that are not present on the response struct will be saved and can be accessed by result.JSON.ExtraFields() which returns the extra fields as a map[string]Field.

Middleware

We provide option.WithMiddleware which applies the given middleware to requests.

func Logger(req *http.Request, next option.MiddlewareNext) (res *http.Response, err error) {
	// Before the request
	start := time.Now()
	LogReq(req)

	// Forward the request to the next handler
	res, err = next(req)

	// Handle stuff after the request
	end := time.Now()
	LogRes(res, err, start - end)

    return res, err
}

client := rails.NewClient(
	option.WithMiddleware(Logger),
)

When multiple middlewares are provided as variadic arguments, the middlewares are applied left to right. If option.WithMiddleware is given multiple times, for example first in the client then the method, the middleware in the client will run first and the middleware given in the method will run next.

You may also replace the default http.Client with option.WithHTTPClient(client). Only one http client is accepted (this overwrites any previous client) and receives requests after any middleware has been applied.

Semantic versioning

This package generally follows SemVer conventions, though certain backwards-incompatible changes may be released as minor versions:

  1. Changes to library internals which are technically public but not intended or documented for external use. (Please open a GitHub issue to let us know if you are relying on such internals.)
  2. Changes that we do not expect to impact the vast majority of users in practice.

We take backwards-compatibility seriously and work hard to ensure you can rely on a smooth upgrade experience.

We are keen for your feedback; please open an issue with questions, bugs, or suggestions.

Contributing

See the contributing documentation.

Documentation

Index

Constants

const TransactionStatusCancelled = shared.TransactionStatusCancelled

Equals "cancelled"

const TransactionStatusCompleted = shared.TransactionStatusCompleted

Equals "completed"

const TransactionStatusFailed = shared.TransactionStatusFailed

Equals "failed"

const TransactionStatusPending = shared.TransactionStatusPending

Equals "pending"

const TransactionTransactionTypeDeposit = shared.TransactionTransactionTypeDeposit

Equals "deposit"

const TransactionTransactionTypeRecurringPayment = shared.TransactionTransactionTypeRecurringPayment

Equals "recurring_payment"

const TransactionTransactionTypeSavingsWithdraw = shared.TransactionTransactionTypeSavingsWithdraw

Equals "savings_withdraw"

const TransactionTransactionTypeTransfer = shared.TransactionTransactionTypeTransfer

Equals "transfer"

const TransactionTransactionTypeWithdrawal = shared.TransactionTransactionTypeWithdrawal

Equals "withdrawal"

Variables

This section is empty.

Functions

func Bool

func Bool(b bool) param.Opt[bool]

func BoolPtr

func BoolPtr(v bool) *bool

func DefaultClientOptions

func DefaultClientOptions() []option.RequestOption

DefaultClientOptions read from the environment (RAILS_API_KEY, RAILS_BASE_URL). This should be used to initialize new clients.

func File

func File(rdr io.Reader, filename string, contentType string) file

func Float

func Float(f float64) param.Opt[float64]

func FloatPtr

func FloatPtr(v float64) *float64

func Int

func Int(i int64) param.Opt[int64]

func IntPtr

func IntPtr(v int64) *int64

func Opt

func Opt[T comparable](v T) param.Opt[T]

func Ptr

func Ptr[T any](v T) *T

func String

func String(s string) param.Opt[string]

func StringPtr

func StringPtr(v string) *string

func Time

func Time(t time.Time) param.Opt[time.Time]

func TimePtr

func TimePtr(v time.Time) *time.Time

Types

type Account

type Account struct {
	ID            string `json:"id" api:"required" format:"uuid"`
	AccountNumber string `json:"account_number" api:"required"`
	// Any of "checking", "saving".
	AccountType AccountAccountType `json:"account_type" api:"required"`
	Balance     string             `json:"balance" api:"required"`
	Currency    string             `json:"currency" api:"required"`
	Environment string             `json:"environment" api:"required"`
	// Any of "active", "suspended", "closed".
	Status         AccountStatus `json:"status" api:"required"`
	UserID         string        `json:"user_id" api:"required" format:"uuid"`
	AdminUserID    string        `json:"admin_user_id" api:"nullable" format:"uuid"`
	CreatedAt      time.Time     `json:"created_at" api:"nullable" format:"date-time"`
	OrganizationID string        `json:"organization_id" api:"nullable" format:"uuid"`
	UpdatedAt      time.Time     `json:"updated_at" api:"nullable" format:"date-time"`
	UserRole       string        `json:"user_role" api:"nullable"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		ID             respjson.Field
		AccountNumber  respjson.Field
		AccountType    respjson.Field
		Balance        respjson.Field
		Currency       respjson.Field
		Environment    respjson.Field
		Status         respjson.Field
		UserID         respjson.Field
		AdminUserID    respjson.Field
		CreatedAt      respjson.Field
		OrganizationID respjson.Field
		UpdatedAt      respjson.Field
		UserRole       respjson.Field
		ExtraFields    map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (Account) RawJSON

func (r Account) RawJSON() string

Returns the unmodified JSON received from the API

func (*Account) UnmarshalJSON

func (r *Account) UnmarshalJSON(data []byte) error

type AccountAccountType

type AccountAccountType string
const (
	AccountAccountTypeChecking AccountAccountType = "checking"
	AccountAccountTypeSaving   AccountAccountType = "saving"
)

type AccountDepositParams

type AccountDepositParams struct {
	Amount      string            `json:"amount" api:"required"`
	Description param.Opt[string] `json:"description,omitzero"`
	// contains filtered or unexported fields
}

func (AccountDepositParams) MarshalJSON

func (r AccountDepositParams) MarshalJSON() (data []byte, err error)

func (*AccountDepositParams) UnmarshalJSON

func (r *AccountDepositParams) UnmarshalJSON(data []byte) error

type AccountDepositResponse

type AccountDepositResponse struct {
	Account     Account            `json:"account" api:"required"`
	Transaction shared.Transaction `json:"transaction" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Account     respjson.Field
		Transaction respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AccountDepositResponse) RawJSON

func (r AccountDepositResponse) RawJSON() string

Returns the unmodified JSON received from the API

func (*AccountDepositResponse) UnmarshalJSON

func (r *AccountDepositResponse) UnmarshalJSON(data []byte) error

type AccountListParams

type AccountListParams struct {
	UserID string `query:"user_id" api:"required" format:"uuid" json:"-"`
	// contains filtered or unexported fields
}

func (AccountListParams) URLQuery

func (r AccountListParams) URLQuery() (v url.Values, err error)

URLQuery serializes AccountListParams's query parameters as `url.Values`.

type AccountNewParams

type AccountNewParams struct {
	// Any of "checking", "saving".
	AccountType AccountNewParamsAccountType `json:"account_type,omitzero" api:"required"`
	// Three-letter uppercase ISO currency code, for example USD or ZAR.
	Currency       string            `json:"currency" api:"required"`
	UserID         string            `json:"user_id" api:"required" format:"uuid"`
	Environment    param.Opt[string] `json:"environment,omitzero"`
	OrganizationID param.Opt[string] `json:"organization_id,omitzero" format:"uuid"`
	// contains filtered or unexported fields
}

func (AccountNewParams) MarshalJSON

func (r AccountNewParams) MarshalJSON() (data []byte, err error)

func (*AccountNewParams) UnmarshalJSON

func (r *AccountNewParams) UnmarshalJSON(data []byte) error

type AccountNewParamsAccountType

type AccountNewParamsAccountType string
const (
	AccountNewParamsAccountTypeChecking AccountNewParamsAccountType = "checking"
	AccountNewParamsAccountTypeSaving   AccountNewParamsAccountType = "saving"
)

type AccountService

type AccountService struct {
	Options []option.RequestOption
}

Accounts

AccountService contains methods and other services that help with interacting with the rails API.

Note, unlike clients, this service does not read variables from the environment automatically. You should not instantiate this service directly, and instead use the NewAccountService method instead.

func NewAccountService

func NewAccountService(opts ...option.RequestOption) (r AccountService)

NewAccountService generates a new service that applies the given options to each request. These options are applied after the parent client's options (if there is one), and before any request-specific options.

func (*AccountService) Close

func (r *AccountService) Close(ctx context.Context, id string, opts ...option.RequestOption) (res *Account, err error)

Close account

func (*AccountService) Deposit

Deposit into account

func (*AccountService) Get

func (r *AccountService) Get(ctx context.Context, id string, opts ...option.RequestOption) (res *Account, err error)

Retrieve account

func (*AccountService) List

func (r *AccountService) List(ctx context.Context, query AccountListParams, opts ...option.RequestOption) (res *[]Account, err error)

List accounts

func (*AccountService) New

func (r *AccountService) New(ctx context.Context, body AccountNewParams, opts ...option.RequestOption) (res *Account, err error)

Create account

func (*AccountService) Transfer

Transfer between accounts

func (*AccountService) UpdateStatus

func (r *AccountService) UpdateStatus(ctx context.Context, id string, body AccountUpdateStatusParams, opts ...option.RequestOption) (res *Account, err error)

Update account status

func (*AccountService) Withdraw

Withdraw from account

type AccountStatus

type AccountStatus string
const (
	AccountStatusActive    AccountStatus = "active"
	AccountStatusSuspended AccountStatus = "suspended"
	AccountStatusClosed    AccountStatus = "closed"
)

type AccountTransferParams

type AccountTransferParams struct {
	Amount      string            `json:"amount" api:"required"`
	ToAccountID string            `json:"to_account_id" api:"required" format:"uuid"`
	Description param.Opt[string] `json:"description,omitzero"`
	// contains filtered or unexported fields
}

func (AccountTransferParams) MarshalJSON

func (r AccountTransferParams) MarshalJSON() (data []byte, err error)

func (*AccountTransferParams) UnmarshalJSON

func (r *AccountTransferParams) UnmarshalJSON(data []byte) error

type AccountTransferResponse

type AccountTransferResponse struct {
	FromAccount Account            `json:"from_account" api:"required"`
	ToAccount   Account            `json:"to_account" api:"required"`
	Transaction shared.Transaction `json:"transaction" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		FromAccount respjson.Field
		ToAccount   respjson.Field
		Transaction respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AccountTransferResponse) RawJSON

func (r AccountTransferResponse) RawJSON() string

Returns the unmodified JSON received from the API

func (*AccountTransferResponse) UnmarshalJSON

func (r *AccountTransferResponse) UnmarshalJSON(data []byte) error

type AccountUpdateStatusParams

type AccountUpdateStatusParams struct {
	// Any of "active", "suspended", "closed".
	Status AccountUpdateStatusParamsStatus `json:"status,omitzero"`
	// contains filtered or unexported fields
}

func (AccountUpdateStatusParams) MarshalJSON

func (r AccountUpdateStatusParams) MarshalJSON() (data []byte, err error)

func (*AccountUpdateStatusParams) UnmarshalJSON

func (r *AccountUpdateStatusParams) UnmarshalJSON(data []byte) error

type AccountUpdateStatusParamsStatus

type AccountUpdateStatusParamsStatus string
const (
	AccountUpdateStatusParamsStatusActive    AccountUpdateStatusParamsStatus = "active"
	AccountUpdateStatusParamsStatusSuspended AccountUpdateStatusParamsStatus = "suspended"
	AccountUpdateStatusParamsStatusClosed    AccountUpdateStatusParamsStatus = "closed"
)

type AccountWithdrawParams

type AccountWithdrawParams struct {
	Amount      string            `json:"amount" api:"required"`
	Description param.Opt[string] `json:"description,omitzero"`
	// contains filtered or unexported fields
}

func (AccountWithdrawParams) MarshalJSON

func (r AccountWithdrawParams) MarshalJSON() (data []byte, err error)

func (*AccountWithdrawParams) UnmarshalJSON

func (r *AccountWithdrawParams) UnmarshalJSON(data []byte) error

type AccountWithdrawResponse

type AccountWithdrawResponse struct {
	Account     Account            `json:"account" api:"required"`
	Transaction shared.Transaction `json:"transaction" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Account     respjson.Field
		Transaction respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AccountWithdrawResponse) RawJSON

func (r AccountWithdrawResponse) RawJSON() string

Returns the unmodified JSON received from the API

func (*AccountWithdrawResponse) UnmarshalJSON

func (r *AccountWithdrawResponse) UnmarshalJSON(data []byte) error

type AuditEventListParams added in v0.3.0

type AuditEventListParams struct {
	Action     param.Opt[string]    `query:"action,omitzero" json:"-"`
	From       param.Opt[time.Time] `query:"from,omitzero" format:"date-time" json:"-"`
	Page       param.Opt[int64]     `query:"page,omitzero" json:"-"`
	PerPage    param.Opt[int64]     `query:"per_page,omitzero" json:"-"`
	TargetID   param.Opt[string]    `query:"target_id,omitzero" json:"-"`
	TargetType param.Opt[string]    `query:"target_type,omitzero" json:"-"`
	To         param.Opt[time.Time] `query:"to,omitzero" format:"date-time" json:"-"`
	// Environment to list audit events from. Defaults to sandbox when omitted.
	//
	// Any of "sandbox", "production".
	Environment AuditEventListParamsEnvironment `query:"environment,omitzero" json:"-"`
	// Any of "success", "client_error", "server_error".
	Outcome AuditEventListParamsOutcome `query:"outcome,omitzero" json:"-"`
	// contains filtered or unexported fields
}

func (AuditEventListParams) URLQuery added in v0.3.0

func (r AuditEventListParams) URLQuery() (v url.Values, err error)

URLQuery serializes AuditEventListParams's query parameters as `url.Values`.

type AuditEventListParamsEnvironment added in v0.3.0

type AuditEventListParamsEnvironment string

Environment to list audit events from. Defaults to sandbox when omitted.

const (
	AuditEventListParamsEnvironmentSandbox    AuditEventListParamsEnvironment = "sandbox"
	AuditEventListParamsEnvironmentProduction AuditEventListParamsEnvironment = "production"
)

type AuditEventListParamsOutcome added in v0.3.0

type AuditEventListParamsOutcome string
const (
	AuditEventListParamsOutcomeSuccess     AuditEventListParamsOutcome = "success"
	AuditEventListParamsOutcomeClientError AuditEventListParamsOutcome = "client_error"
	AuditEventListParamsOutcomeServerError AuditEventListParamsOutcome = "server_error"
)

type AuditEventListResponse added in v0.3.0

type AuditEventListResponse struct {
	Data       []AuditEventListResponseData     `json:"data" api:"required"`
	Pagination AuditEventListResponsePagination `json:"pagination" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Data        respjson.Field
		Pagination  respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AuditEventListResponse) RawJSON added in v0.3.0

func (r AuditEventListResponse) RawJSON() string

Returns the unmodified JSON received from the API

func (*AuditEventListResponse) UnmarshalJSON added in v0.3.0

func (r *AuditEventListResponse) UnmarshalJSON(data []byte) error

type AuditEventListResponseData added in v0.3.0

type AuditEventListResponseData struct {
	ID string `json:"id" api:"required" format:"uuid"`
	// Any of "users.business.register", "users.auth.login", "users.auth.refresh",
	// "users.auth.revoke", "users.password_reset.request",
	// "users.password_reset.complete", "users.beta.apply", "users.api_key.create",
	// "users.api_key.revoke", "accounts.account.create",
	// "accounts.account.update_status", "accounts.account.close",
	// "accounts.money.deposit", "accounts.money.withdraw", "accounts.money.transfer",
	// "ledger.transaction.post".
	Action        string                          `json:"action" api:"required"`
	Actor         AuditEventListResponseDataActor `json:"actor" api:"required"`
	CorrelationID string                          `json:"correlation_id" api:"required"`
	CreatedAt     time.Time                       `json:"created_at" api:"required" format:"date-time"`
	// Any of "sandbox", "production".
	Environment    string            `json:"environment" api:"required"`
	Metadata       map[string]string `json:"metadata" api:"required"`
	OccurredAt     time.Time         `json:"occurred_at" api:"required" format:"date-time"`
	OrganizationID string            `json:"organization_id" api:"required" format:"uuid"`
	// Any of "success", "client_error", "server_error".
	Outcome string                            `json:"outcome" api:"required"`
	Request AuditEventListResponseDataRequest `json:"request" api:"required"`
	// Any of 1.
	SchemaVersion int64 `json:"schema_version" api:"required"`
	// Any of "users", "accounts", "ledger".
	SourceService string                           `json:"source_service" api:"required"`
	Target        AuditEventListResponseDataTarget `json:"target" api:"required"`
	Reason        string                           `json:"reason" api:"nullable"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		ID             respjson.Field
		Action         respjson.Field
		Actor          respjson.Field
		CorrelationID  respjson.Field
		CreatedAt      respjson.Field
		Environment    respjson.Field
		Metadata       respjson.Field
		OccurredAt     respjson.Field
		OrganizationID respjson.Field
		Outcome        respjson.Field
		Request        respjson.Field
		SchemaVersion  respjson.Field
		SourceService  respjson.Field
		Target         respjson.Field
		Reason         respjson.Field
		ExtraFields    map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AuditEventListResponseData) RawJSON added in v0.3.0

func (r AuditEventListResponseData) RawJSON() string

Returns the unmodified JSON received from the API

func (*AuditEventListResponseData) UnmarshalJSON added in v0.3.0

func (r *AuditEventListResponseData) UnmarshalJSON(data []byte) error

type AuditEventListResponseDataActor added in v0.3.0

type AuditEventListResponseDataActor struct {
	ID string `json:"id" api:"required"`
	// Any of "user", "api_key", "internal_service", "anonymous".
	Type  string   `json:"type" api:"required"`
	Roles []string `json:"roles"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		ID          respjson.Field
		Type        respjson.Field
		Roles       respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AuditEventListResponseDataActor) RawJSON added in v0.3.0

Returns the unmodified JSON received from the API

func (*AuditEventListResponseDataActor) UnmarshalJSON added in v0.3.0

func (r *AuditEventListResponseDataActor) UnmarshalJSON(data []byte) error

type AuditEventListResponseDataRequest added in v0.3.0

type AuditEventListResponseDataRequest struct {
	ID        string `json:"id" api:"required"`
	Method    string `json:"method" api:"required"`
	Path      string `json:"path" api:"required"`
	IP        string `json:"ip"`
	UserAgent string `json:"user_agent"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		ID          respjson.Field
		Method      respjson.Field
		Path        respjson.Field
		IP          respjson.Field
		UserAgent   respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AuditEventListResponseDataRequest) RawJSON added in v0.3.0

Returns the unmodified JSON received from the API

func (*AuditEventListResponseDataRequest) UnmarshalJSON added in v0.3.0

func (r *AuditEventListResponseDataRequest) UnmarshalJSON(data []byte) error

type AuditEventListResponseDataTarget added in v0.3.0

type AuditEventListResponseDataTarget struct {
	ID   string `json:"id" api:"required"`
	Type string `json:"type" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		ID          respjson.Field
		Type        respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AuditEventListResponseDataTarget) RawJSON added in v0.3.0

Returns the unmodified JSON received from the API

func (*AuditEventListResponseDataTarget) UnmarshalJSON added in v0.3.0

func (r *AuditEventListResponseDataTarget) UnmarshalJSON(data []byte) error

type AuditEventListResponsePagination added in v0.3.0

type AuditEventListResponsePagination struct {
	Page       int64 `json:"page" api:"required"`
	PerPage    int64 `json:"per_page" api:"required"`
	TotalCount int64 `json:"total_count" api:"required"`
	TotalPages int64 `json:"total_pages" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Page        respjson.Field
		PerPage     respjson.Field
		TotalCount  respjson.Field
		TotalPages  respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (AuditEventListResponsePagination) RawJSON added in v0.3.0

Returns the unmodified JSON received from the API

func (*AuditEventListResponsePagination) UnmarshalJSON added in v0.3.0

func (r *AuditEventListResponsePagination) UnmarshalJSON(data []byte) error

type AuditEventService added in v0.3.0

type AuditEventService struct {
	Options []option.RequestOption
}

Audit events

AuditEventService contains methods and other services that help with interacting with the rails API.

Note, unlike clients, this service does not read variables from the environment automatically. You should not instantiate this service directly, and instead use the NewAuditEventService method instead.

func NewAuditEventService added in v0.3.0

func NewAuditEventService(opts ...option.RequestOption) (r AuditEventService)

NewAuditEventService generates a new service that applies the given options to each request. These options are applied after the parent client's options (if there is one), and before any request-specific options.

func (*AuditEventService) List added in v0.3.0

List audit events

type Client

type Client struct {
	Options []option.RequestOption
	// Users
	Users UserService
	// Accounts
	Accounts AccountService
	// Transactions
	Transactions TransactionService
	// Audit events
	AuditEvents AuditEventService
}

Client creates a struct with services and top level methods that help with interacting with the rails API. You should not instantiate this client directly, and instead use the NewClient method instead.

func NewClient

func NewClient(opts ...option.RequestOption) (r Client)

NewClient generates a new client with the default option read from the environment (RAILS_API_KEY, RAILS_BASE_URL). The option passed in as arguments are applied after these default arguments, and all option will be passed down to the services and requests that this client makes.

func (*Client) Delete

func (r *Client) Delete(ctx context.Context, path string, params any, res any, opts ...option.RequestOption) error

Delete makes a DELETE request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Execute

func (r *Client) Execute(ctx context.Context, method string, path string, params any, res any, opts ...option.RequestOption) error

Execute makes a request with the given context, method, URL, request params, response, and request options. This is useful for hitting undocumented endpoints while retaining the base URL, auth, retries, and other options from the client.

If a byte slice or an io.Reader is supplied to params, it will be used as-is for the request body.

The params is by default serialized into the body using encoding/json. If your type implements a MarshalJSON function, it will be used instead to serialize the request. If a URLQuery method is implemented, the returned url.Values will be used as query strings to the url.

If your params struct uses param.Field, you must provide either [MarshalJSON], [URLQuery], and/or [MarshalForm] functions. It is undefined behavior to use a struct uses param.Field without specifying how it is serialized.

Any "…Params" object defined in this library can be used as the request argument. Note that 'path' arguments will not be forwarded into the url.

The response body will be deserialized into the res variable, depending on its type:

  • A pointer to a *http.Response is populated by the raw response.
  • A pointer to a byte array will be populated with the contents of the request body.
  • A pointer to any other type uses this library's default JSON decoding, which respects UnmarshalJSON if it is defined on the type.
  • A nil value will not read the response body.

For even greater flexibility, see option.WithResponseInto and option.WithResponseBodyInto.

func (*Client) Get

func (r *Client) Get(ctx context.Context, path string, params any, res any, opts ...option.RequestOption) error

Get makes a GET request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Patch

func (r *Client) Patch(ctx context.Context, path string, params any, res any, opts ...option.RequestOption) error

Patch makes a PATCH request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Post

func (r *Client) Post(ctx context.Context, path string, params any, res any, opts ...option.RequestOption) error

Post makes a POST request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

func (*Client) Put

func (r *Client) Put(ctx context.Context, path string, params any, res any, opts ...option.RequestOption) error

Put makes a PUT request with the given URL, params, and optionally deserializes to a response. See [Execute] documentation on the params and response.

type Error

type Error = apierror.Error

type Transaction

type Transaction = shared.Transaction

This is an alias to an internal type.

type TransactionListByAccountParams

type TransactionListByAccountParams struct {
	Limit param.Opt[int64] `query:"limit,omitzero" json:"-"`
	// contains filtered or unexported fields
}

func (TransactionListByAccountParams) URLQuery

func (r TransactionListByAccountParams) URLQuery() (v url.Values, err error)

URLQuery serializes TransactionListByAccountParams's query parameters as `url.Values`.

type TransactionListParams

type TransactionListParams struct {
	OrganizationID string           `query:"organization_id" api:"required" format:"uuid" json:"-"`
	Page           param.Opt[int64] `query:"page,omitzero" json:"-"`
	PerPage        param.Opt[int64] `query:"per_page,omitzero" json:"-"`
	// contains filtered or unexported fields
}

func (TransactionListParams) URLQuery

func (r TransactionListParams) URLQuery() (v url.Values, err error)

URLQuery serializes TransactionListParams's query parameters as `url.Values`.

type TransactionListResponse

type TransactionListResponse struct {
	Data       []TransactionListResponseData     `json:"data" api:"required"`
	Pagination TransactionListResponsePagination `json:"pagination" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Data        respjson.Field
		Pagination  respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (TransactionListResponse) RawJSON

func (r TransactionListResponse) RawJSON() string

Returns the unmodified JSON received from the API

func (*TransactionListResponse) UnmarshalJSON

func (r *TransactionListResponse) UnmarshalJSON(data []byte) error

type TransactionListResponseData

type TransactionListResponseData struct {
	ID string `json:"id" api:"required" format:"uuid"`
	// Amount in minor units
	Amount         int64     `json:"amount" api:"required"`
	CreatedAt      time.Time `json:"created_at" api:"required" format:"date-time"`
	Currency       string    `json:"currency" api:"required"`
	FromAccountID  string    `json:"from_account_id" api:"required" format:"uuid"`
	OrganizationID string    `json:"organization_id" api:"required" format:"uuid"`
	// Any of "pending", "posted", "failed".
	Status      string `json:"status" api:"required"`
	ToAccountID string `json:"to_account_id" api:"required" format:"uuid"`
	// Any of "deposit", "withdraw", "transfer".
	TransactionKind string    `json:"transaction_kind" api:"required"`
	UpdatedAt       time.Time `json:"updated_at" api:"required" format:"date-time"`
	Environment     string    `json:"environment" api:"nullable"`
	FailureReason   string    `json:"failure_reason" api:"nullable"`
	IdempotencyKey  string    `json:"idempotency_key"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		ID              respjson.Field
		Amount          respjson.Field
		CreatedAt       respjson.Field
		Currency        respjson.Field
		FromAccountID   respjson.Field
		OrganizationID  respjson.Field
		Status          respjson.Field
		ToAccountID     respjson.Field
		TransactionKind respjson.Field
		UpdatedAt       respjson.Field
		Environment     respjson.Field
		FailureReason   respjson.Field
		IdempotencyKey  respjson.Field
		ExtraFields     map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

Transaction as returned by list-by-organization (organization_id, from/to_account_id, transaction_kind).

func (TransactionListResponseData) RawJSON

func (r TransactionListResponseData) RawJSON() string

Returns the unmodified JSON received from the API

func (*TransactionListResponseData) UnmarshalJSON

func (r *TransactionListResponseData) UnmarshalJSON(data []byte) error

type TransactionListResponsePagination

type TransactionListResponsePagination struct {
	Page       int64 `json:"page" api:"required"`
	PerPage    int64 `json:"per_page" api:"required"`
	TotalCount int64 `json:"total_count" api:"required"`
	TotalPages int64 `json:"total_pages" api:"required"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Page        respjson.Field
		PerPage     respjson.Field
		TotalCount  respjson.Field
		TotalPages  respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (TransactionListResponsePagination) RawJSON

Returns the unmodified JSON received from the API

func (*TransactionListResponsePagination) UnmarshalJSON

func (r *TransactionListResponsePagination) UnmarshalJSON(data []byte) error

type TransactionService

type TransactionService struct {
	Options []option.RequestOption
}

Transactions

TransactionService contains methods and other services that help with interacting with the rails API.

Note, unlike clients, this service does not read variables from the environment automatically. You should not instantiate this service directly, and instead use the NewTransactionService method instead.

func NewTransactionService

func NewTransactionService(opts ...option.RequestOption) (r TransactionService)

NewTransactionService generates a new service that applies the given options to each request. These options are applied after the parent client's options (if there is one), and before any request-specific options.

func (*TransactionService) Get

func (r *TransactionService) Get(ctx context.Context, id string, opts ...option.RequestOption) (res *shared.Transaction, err error)

Retrieve transaction

func (*TransactionService) List

List transactions by organization

func (*TransactionService) ListByAccount

func (r *TransactionService) ListByAccount(ctx context.Context, accountID string, query TransactionListByAccountParams, opts ...option.RequestOption) (res *[]shared.Transaction, err error)

List account transactions

type TransactionStatus

type TransactionStatus = shared.TransactionStatus

This is an alias to an internal type.

type TransactionTransactionType

type TransactionTransactionType = shared.TransactionTransactionType

This is an alias to an internal type.

type UserNewParams

type UserNewParams struct {
	Email     string `json:"email" api:"required" format:"email"`
	FirstName string `json:"first_name" api:"required"`
	LastName  string `json:"last_name" api:"required"`
	Password  string `json:"password" api:"required" format:"password"`
	// Any of "sandbox", "production".
	XEnvironment UserNewParamsXEnvironment `header:"X-Environment,omitzero" api:"required" json:"-"`
	// contains filtered or unexported fields
}

func (UserNewParams) MarshalJSON

func (r UserNewParams) MarshalJSON() (data []byte, err error)

func (*UserNewParams) UnmarshalJSON

func (r *UserNewParams) UnmarshalJSON(data []byte) error

type UserNewParamsXEnvironment

type UserNewParamsXEnvironment string
const (
	UserNewParamsXEnvironmentSandbox    UserNewParamsXEnvironment = "sandbox"
	UserNewParamsXEnvironmentProduction UserNewParamsXEnvironment = "production"
)

type UserNewResponse

type UserNewResponse struct {
	Status string `json:"status" api:"required"`
	UserID string `json:"user_id" api:"required" format:"uuid"`
	// JSON contains metadata for fields, check presence with [respjson.Field.Valid].
	JSON struct {
		Status      respjson.Field
		UserID      respjson.Field
		ExtraFields map[string]respjson.Field
		// contains filtered or unexported fields
	} `json:"-"`
}

func (UserNewResponse) RawJSON

func (r UserNewResponse) RawJSON() string

Returns the unmodified JSON received from the API

func (*UserNewResponse) UnmarshalJSON

func (r *UserNewResponse) UnmarshalJSON(data []byte) error

type UserService

type UserService struct {
	Options []option.RequestOption
}

Users

UserService contains methods and other services that help with interacting with the rails API.

Note, unlike clients, this service does not read variables from the environment automatically. You should not instantiate this service directly, and instead use the NewUserService method instead.

func NewUserService

func NewUserService(opts ...option.RequestOption) (r UserService)

NewUserService generates a new service that applies the given options to each request. These options are applied after the parent client's options (if there is one), and before any request-specific options.

func (*UserService) New

func (r *UserService) New(ctx context.Context, params UserNewParams, opts ...option.RequestOption) (res *UserNewResponse, err error)

Create user

Directories

Path Synopsis
encoding/json
Package json implements encoding and decoding of JSON as defined in RFC 7159.
Package json implements encoding and decoding of JSON as defined in RFC 7159.
encoding/json/shims
This package provides shims over Go 1.2{2,3} APIs which are missing from Go 1.22, and used by the Go 1.24 encoding/json package.
This package provides shims over Go 1.2{2,3} APIs which are missing from Go 1.22, and used by the Go 1.24 encoding/json package.
packages

Jump to